all: libimms.so

libimms.so: $(OBJECTS) immsconf.h
	$(CXX) $(LDFLAGS) \
        $(OBJECTS) \
        -shared -Wl,-soname,$@ -o $@

%.o: %.cc vars.mk
	@$(CXX) $(CFLAGS) -M -E $< > .$*.d
	$(CXX) $(CFLAGS) -c $< -o $@

%.o: %.c vars.mk
	@$(CXX) $(CFLAGS) -M -E $< > .$*.d
	$(CC) $(CFLAGS) -c $< -o $@

.PHONY: clean distclean

clean:
	rm -f $(wildcard $(OBJECTS) libimms.so imms-*.tar.* core* .*.d)

distclean: clean
	rm -f $(wildcard .\#* config.* configure immsconf.h* vars.mk)
	rm -rf $(wildcard autom4te.cache)

.PHONY: dist

configure: configure.ac vars.mk.in
	autoheader
	autoconf

immsconf.h: configure
	$(error Please run the "configure" script)

dist: immsconf.h distclean 
	cp -r . /tmp/imms-$(VERSION)
	rm -rf /tmp/imms-$(VERSION)/CVS /tmp/imms-$(VERSION)/.cvsignore
	tar -C /tmp/ -cj imms-$(VERSION)/ -f imms-$(VERSION).tar.bz2
	tar -C /tmp/ -cz imms-$(VERSION)/ -f imms-$(VERSION).tar.gz
	rm -rf /tmp/imms-$(VERSION)/

.PHONY: install install-user install-system user-message system-message

ifeq ($(shell id -u), 0)
install: system-message install-system
else
install: user-message install-user
endif

system-message:
	$(warning Defaulting to installing for all users.)
	$(warning Use 'make install-user' to install for the current user only.)

install-system: libimms.so
	${INSTALL_PROGRAM} libimms.so `xmms-config --general-plugin-dir`

user-message:
	$(warning Defaulting to installing for current user only.)
	$(warning Use 'make install-system' to install for all users.)

install-user: libimms.so
	mkdir -p ~/.xmms/Plugins/General/
	${INSTALL_PROGRAM} libimms.so ~/.xmms/Plugins/General/
